# IMI住所変換コンポーネント

JSON-LD に含まれる「表記を持つ住所型」の表記を解析して、
都道府県・市区町村・町名・番地・号を付与するライブラリです。
コマンドラインツールおよび WebAPI サーバも提供されます。

# 使用データについて

本モジュールに同梱している辞書データは [統計に用いる標準地域コード](http://data.e-stat.go.jp/lodw/provdata/lodRegion/#3-2-1)
および
[位置参照情報ダウンロードサービス](http://nlftp.mlit.go.jp/isj/)
のデータを加工して作成したものです。


# 利用者向け情報


## インストール

適当な作業フォルダを作成しモジュールをインストールします。

```
$ mkdir work
$ cd work
$ npm install imi-enrichment-address-1.0.0.tgz
```

## API

モジュール imi-enrichment-address は以下のような API の関数です。

```
module.exports = function(jsonld) {..}
```

- jsonld : 変換対象となる JSON-LD オブジェクト

## 使用例

以下の手順では enrich 関数によって input (JSON) から output (JSON) に変換され、
結果が標準出力に出力されます。

```main.js
const enrich = require('imi-enrichment-address');

const input = {
  "@context": "https://imi.go.jp/ns/core/context.jsonld",
  "@type": "住所型",
  "表記": "千代田区霞が関1-3-1"
};

const output = enrich(input);

console.log(JSON.stringify(output,null,2));
```

実行すると以下のような出力が得られます。

```
$ node main.js
{
  "@context": [
    "https://imi.go.jp/ns/core/context.jsonld",
    "http://hojin-info.go.jp/ns/domain/biz/1/context.jsonld"
  ],
  "@type": "住所型",
  "表記": "千代田区霞が関1-3-1",
  "都道府県": "東京都",
  "市区町村": "千代田区",
  "町名": "霞が関",
  "丁目": "1",
  "番地": "3",
  "号": "1",
  "メタデータ": {
    "@type": "hj:更新履歴型",
    "hj:更新者": {
      "@type": "実体型",
      "表記": "IMI住所変換コンポーネント"
    },
    "種別": "INFO",
    "説明": "プロパティ(都道府県|市区町村|町名|丁目|番地|号) が追加され、都道府県の値が補完されました",
    "日時": {
      "@type": "日時型",
      "標準型日時": "2019-02-08T17:07:41.309+09:00"
    }
  }
}
$
```

## コマンドラインツールとしての利用

コマンドラインツールの実体は `bin/cli.js` です。
指定されたひとつのファイルまたは標準入力に含まれる JSON をパースして
変換結果を標準出力に出力します。

以下は動作確認用ファイル `example/simple.json` を変換する事例です。

```
$ npm install imi-enrichment-address-1.0.0.tgz
$ cd node_modules/imi-enrichment-address
$ node bin/cli.js example/simple.json
{
  "@context": [
    "https://imi.go.jp/ns/core/context.jsonld",
    "http://hojin-info.go.jp/ns/domain/biz/1/context.jsonld"
  ],
  "@type": "住所型",
  "表記": "千代田区霞が関1-3-1",
  "都道府県": "東京都",
  "市区町村": "千代田区",
  "町名": "霞が関",
  "丁目": "1",
  "番地": "3",
  "号": "1",
  "メタデータ": {
    "@type": "hj:更新履歴型",
    "hj:更新者": {
      "@type": "実体型",
      "表記": "IMI住所変換コンポーネント"
    },
    "種別": "INFO",
    "説明": "プロパティ(都道府県|市区町村|町名|丁目|番地|号) が追加され、都道府県の値が補完されました",
    "日時": {
      "@type": "日時型",
      "標準型日時": "2019-02-08T17:07:41.309+09:00"
    }
  }
}

$
```

動作確認用ファイルは以下の３つです。

- example/simple.json : 単一のインスタンスをもつ JSON-LD
- example/graph.json : ３つのインスタンスをもつ JSON-LD
- example/line-delimited.json.txt : 1行ごとに独立した JSON-LD が記録された Line delimited JSON

なお、入力形式が Line delimited JSON と判定された場合には、
出力形式も Line delimited JSON になります。

## サーバとしての利用

コマンドラインツールの実体は `bin/server.js` です。
以下の手順で任意のポートでサーバを起動します。

```
$ npm install imi-enrichment-address-1.0.0.tgz
$ node_modules/imi-enrichment-address
$ node bin/server.js 3030
imi-enrichment-address-server is running on port 3030
```

なお、ポート番号を指定しなかった場合には以下のように使用方法が出力されて終了します。

```
$ node bin/server.js
Usage: node server.js [port number]
$
```

サーバが起動している状態で、 POST メソッドで JSON-LD を送信すると変換結果が出力されます。

```
$ curl -X POST -d '{"@context": "https://imi.go.jp/ns/core/context.jsonld","@type": "住所型","表記": "千代田区霞が関1-3-1"}' localhost:3030
{
  "@context": [
    "https://imi.go.jp/ns/core/context.jsonld",
    "http://hojin-info.go.jp/ns/domain/biz/1/context.jsonld"
  ],
  "@type": "住所型",
  "表記": "千代田区霞が関1-3-1",
  "都道府県": "東京都",
  "市区町村": "千代田区",
  "町名": "霞が関",
  "丁目": "1",
  "番地": "3",
  "号": "1",
  "メタデータ": {
    "@type": "hj:更新履歴型",
    "hj:更新者": {
      "@type": "実体型",
      "表記": "IMI住所変換コンポーネント"
    },
    "種別": "INFO",
    "説明": "プロパティ(都道府県|市区町村|町名|丁目|番地|号) が追加され、都道府県の値が補完されました",
    "日時": {
      "@type": "日時型",
      "標準型日時": "2019-02-08T17:25:00.143+09:00"
    }
  }
}
$
```

# 開発者向け情報

## 環境構築

ソースアーカイブ `imi-enrichment-address-1.0.0.src.tgz` を入手して、
以下の手順でインストールします。

```
$ mkdir imi-enrichment-address
$ cd imi-enrichment-address
$ tar xvzf imi-enrichment-address-1.0.0.src.tgz
$ npm install
```

## テスト

以下の手順でテストが実行され、テスト結果が出力されます。

```
$ cd imi-enrichment-address
$ npm test

> imi-enrichment-address@1.0.0 test imi-enrichment-address
> mocha test/test-*.js

  imi-enrichment-address
    基本動作
      正常系
        ✓ ルートにある住所型インスタンスの変換
        ✓ ルートにない住所型インスタンスの変換
        ✓ @graph による複数インスタンスの変換
        ✓ hj:住所型インスタンスの変換
      異常系
        ✓ ルートオブジェクト配下に住所型が見つからない場合は警告
        ✓ @graph の直下および配下に住所型が見つからない場合は警告
  (省略)

  105 passing (138ms)

$

```

## ベンチマーク

以下の手順で大規模データ(法人インフォダウンロードデータから抽出した住所文字列14万件)が変換され、
結果が標準出力およびファイルに記録されます。

```
$ cd imi-enrichment-address
$ npm run stat

> imi-enrichment-address@1.0.0 stat imi-enrichment-address
> mocha test/stat.js

  imi-enrichment-address
    stat
      全体
        ✓ 入力件数: 147756 (100.00%)
        ✓ 実行時間: 19296 [msec]
        ✓ 10,000件あたり: 1305.94 [msec]
  (省略)

  145 passing (123ms)
$
```

入力データの本体は `test/address.txt` です。
出力先のファイルは `test/output.csv` です。

`test/output.csv` は以下のようなレイアウトの CSV です。

```
表記,都道府県,市区町村,区,町名,丁目,番地,号,種別,説明
三重県志摩市阿児町鵜方１４０１番地４,三重県,志摩市,,阿児町鵜方,,1401,4,INFO,プロパティ(都道府県|市区町村|町名|番地|号) が追加されました
三重県志摩市阿児町鵜方２９８２番地の２,三重県,志摩市,,阿児町鵜方,,2982,2,INFO,プロパティ(都道府県|市区町村|町名|番地|号) が追加されました
三重県松坂市京町５１０番地,三重県,,,,,,,WARNING,複数の候補があり、特定できませんでした [三重県四日市市京町|三重県松阪市京町|三重県桑名市京町]
三重県松阪市,三重県,松阪市,,,,,,INFO,プロパティ(都道府県|市区町村) が追加されました
```

BOM つき UTF-8 でエンコードされており、Excel で開くことで、実際にどのような変換が行われたのかを目視確認することができます。


## ファイル構成

開発対象となる Javascript は以下の３つです。

```
main.js : 関数本体
lib/dictionary.js : 辞書データ
lib/util.js : 住所に用いられる各種数字 (全角数字・半角数字・漢数字) の相互変換ユーティリティ
```

このうち、 `lib/dictionary.js` は次の手順で生成されます。


## 辞書の更新

`lib/dictionary.js` は辞書データです。
「統計に用いる標準地域コード」 および「位置参照情報ダウンロードサービス」のデータ更新があった場合には
以下の手順を実行することで辞書データを更新することができます。

### Step 1. 統計に用いる標準地域コード

以下のコマンドを実行することで、統計局の SPARQL Endpoint から標準地域コード情報が取得され、sac.json ファイルに保存されます。
市区町村の追加更新があった場合に実行が必要です。

```
$ cd data
$ bash sac.sh
// sac.json が生成される
```


### Step 2. 位置参照情報ダウンロードサービス

以下のコマンドを実行することで、位置参照情報ダウンロードサービスから「大字・町丁目レベル位置参照情報」のデータがダウンロードされます。

```
$ cd data
$ bash isj.sh
// isj.csv が生成される
```

なお、isj.sh から参照されている isj.txt には以下のような URL のリストが含まれています。

```
http://nlftp.mlit.go.jp/isj/dls/data/11.0b/01000-11.0b.zip
http://nlftp.mlit.go.jp/isj/dls/data/11.0b/02000-11.0b.zip
http://nlftp.mlit.go.jp/isj/dls/data/11.0b/03000-11.0b.zip
http://nlftp.mlit.go.jp/isj/dls/data/11.0b/04000-11.0b.zip
...
http://nlftp.mlit.go.jp/isj/dls/data/11.0b/47000-11.0b.zip
```

「大字・町丁目レベル位置参照情報」のデータ更新があった場合にはこの URL リストの更新が必要です。


### Step 3. dictionary.js の生成

以下のコマンドを実行することで、上記ふたつのデータから辞書が作成されます

```
$ npm run build-dictionary
// lib/dictionary.js が生成される
```
